#include "fitness.hpp"

#include <fstream>
#include <map>
#include <numeric>
#include <cmath>
#include <iostream>

class Quadgram {
public:
    double score(const std::string& text)
    {
        double score = 0;
        for (int i = 0; i < text.size() - L_ + 1; i++)
        {
            std::string part = text.substr(i, L_);
            auto iterator = ngrams_.find(part);
            if (iterator != ngrams_.end())
            {
                score += iterator->second;
            }
            else
            {
                score += floor_;
            }
        }
        return score;
    }

    static Quadgram& instance()
    {
        static Quadgram instance;
        return instance;
    }

private:
    Quadgram()
    {
        std::ifstream file("german_quadgrams.txt");
        std::string str;
        while(std::getline(file, str))
        {
            int spacePos = str.find_first_of(' ');
            std::string key = str.substr(0, spacePos);
            double count = std::stod(str.substr(spacePos));
            N_ += static_cast<int>(count);
            ngrams_[key] = count;
        }
        L_ = 4;

        for (auto& item : ngrams_)
        {
            ngrams_[item.first] = std::log10(item.second / N_);
        }
        floor_ = std::log10(0.01 / N_);
    }

    std::map<std::string, double> ngrams_;
    double floor_;
    int L_;
    int N_{0};
};

double calculate_fitness(const std::string& text)
{
    return Quadgram::instance().score(text);
}
