#include <sstream>
#include <algorithm>
#include <vector>
#include <future>

#include "enigma/enigma.h"

#include "fitness/fitness.hpp"

struct Setting
{
    std::array<int, 3> rotorIndexes;
    std::array<int, 3> rotorPositions;
    std::string text;
    double score;
};

char int_to_char(int c)
{
    return static_cast<char>(c + 'A');
}

std::array<std::string, 8> rotorNames = {"I", "II", "III", "IV", "V", "VI", "VII", "VIII"};

std::ostream &operator<<(std::ostream &s, const Setting &set)
{
    s << rotorNames[set.rotorIndexes[0]] << " " << rotorNames[set.rotorIndexes[1]] << " " << rotorNames[set.rotorIndexes[2]];
    s << " - " << int_to_char(set.rotorPositions[0]) << " " << int_to_char(set.rotorPositions[1]) << " " << int_to_char(set.rotorPositions[2]);
    // s << " - " << set.text;
    return s << " : " << set.score;
}

Setting try_all_keys(int firstRotor, int secondRotor, int thirdRotor, const std::string &cipher)
{
    std::vector<std::string> pairs;
    for (size_t i = 0; i < 10; i++)
    {
        pairs.push_back("AA");
    }

    Plugboard plugboard(pairs);
    Reflector reflector = B;

    Setting best;
    best.score = -999999999999999999;
    best.rotorPositions = {0, 0, 0};
    best.text = "";
    best.rotorIndexes = {firstRotor, secondRotor, thirdRotor};

    for (int i = 0; i < 26; i++)
    {
        for (int j = 0; j < 26; j++)
        {
            for (int k = 0; k < 26; k++)
            {
                Rotor left_rotor = rotors.at(rotorNames[firstRotor]).Duplicate(0, i);
                Rotor center_rotor = rotors.at(rotorNames[secondRotor]).Duplicate(0, j);
                Rotor right_rotor = rotors.at(rotorNames[thirdRotor]).Duplicate(0, k);

                Enigma enigma({left_rotor, center_rotor, right_rotor}, reflector, plugboard);

                std::string result = enigma.encrypt(cipher);
                double cypherIndex = calculate_fitness(result);
                if (cypherIndex > best.score)
                {
                    best.score = cypherIndex;
                    best.rotorPositions = {i, j, k};
                    best.text = result;
                }
            }
        }
    }

    std::cout << best << std::endl;

    return best;
}

bool operator<(const Setting &left, const Setting &right)
{
    return left.score < right.score;
}

void find_rotor_order(const std::string &cipher, int first_disk)
{
    std::vector<std::future<Setting>> asyncResults;
    std::vector<Setting> results;
    // for (int i = 4; i < 8; i++)
    // {
    int i = first_disk;
    for (int j = 0; j < 8; j++)
    {
        if (i == j)
            continue;
        for (int k = 0; k < 8; k++)
        {
            if (k == i || k == j)
                continue;
            if (i < 5 && j < 5 && k < 5)
                continue;
            std::cout << "Trying " << rotorNames[i] << ", " << rotorNames[j] << ", " << rotorNames[k] << std::endl;

            asyncResults.push_back(std::async(try_all_keys, i, j, k, cipher));
        }
    }

    for (auto&& result: asyncResults)
    {
        result.wait();
        results.push_back(result.get());
    }

    std::sort(results.begin(), results.end());

    for (const auto& result: results)
    {
        std::cout << result << std::endl;
    }
}

int main(int argc, char **argv)
{
    std::stringstream sstream;
    sstream << std::cin.rdbuf();
    std::string cipher = sstream.str();

    cleanCipher(cipher);

    if (argc < 2)
    {
        std::cout << "You must provide first motor index";
        return -1;
    }

    find_rotor_order(cipher, std::atoi(argv[1]));
}
